import pathlib

import pytest
import jaraco.path

from jaraco import vcs


def _ensure_present(mgr):
    try:
        mgr.version()
    except Exception:
        pytest.skip()


@pytest.fixture
def temp_work_dir(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    return tmp_path


source_tree = dict(
    bar=dict(
        baz="",
    ),
)


@pytest.fixture
def hg_repo(temp_work_dir):
    mgr = vcs.Mercurial()
    _ensure_present(mgr)
    mgr._invoke('init', '.')
    jaraco.path.build(source_tree)
    mgr._invoke('addremove')
    mgr._invoke('ci', '-m', 'committed')
    pathlib.Path('bar/baz').write_text('content', encoding='utf-8')
    mgr._invoke('ci', '-m', 'added content')


@pytest.fixture
def git_repo(temp_work_dir):
    mgr = vcs.Git()
    _ensure_present(mgr)
    mgr._invoke('init')
    mgr._invoke('config', 'user.email', 'vip@example.com')
    mgr._invoke('config', 'user.name', 'Important User')
    jaraco.path.build(source_tree)
    mgr._invoke('add', '.')
    mgr._invoke('commit', '-m', 'committed')
    pathlib.Path('bar/baz').write_text('content', encoding='utf-8')
    mgr._invoke('commit', '-am', 'added content')



"""
>>> repo().get_current_version()
'...'
>>> isinstance(repo(), Repo)
True
"""

from .base import Repo
from .subprocess import Mercurial, Git


__all__ = ['Repo', 'repo']

repo = Repo.detect

# for compatibility
RepoManager = Repo
MercurialManager = Mercurial
GitManager = Git



import os
import subprocess

from . import base
from . import cmd


class Subprocess:
    env = None

    def _invoke(self, *params):
        """
        Invoke self.exe as a subprocess
        """
        cmd = [self.exe] + list(params)
        proc = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=self.location,
            env=self.env,
        )
        stdout, stderr = proc.communicate()
        if not proc.returncode == 0:
            raise RuntimeError(stderr.strip() or stdout.strip())
        return stdout.decode('utf-8')


class Mercurial(Subprocess, cmd.Mercurial, base.Repo):
    """
    A Repo implemented by calling into the 'hg' command-line
    as a subprocess.
    """

    priority = 1 + os.path.isdir('.hg')

    @property
    def env(self):
        """
        Return an environment safe for calling an `hg` subprocess.

        Removes MACOSX_DEPLOYMENT_TARGET from the env, as if there's a
        mismatch between the local Python environment and the environment
        in which `hg` is installed, it will cause an exception. See
        https://github.com/jaraco/jaraco.vcs/issues/7 for details.
        """
        env = os.environ.copy()
        env.pop('MACOSX_DEPLOYMENT_TARGET', None)
        return env


class Git(Subprocess, cmd.Git, base.Repo):
    """
    A RepoManager implemented by calling into the 'git' command-line
    as a subprocess.
    """

    priority = 1 + os.path.isdir('.git')
